%#########################################################################%
%	Numerics of Partial Differential Equations, Sample Exercise		      %
%	University of Konstanz, Winter semester 2017/2018, Prof. Dr. Volkwein %
%   Authors: Luca Mechelli, Jianjie Lu, {luca.mechelli,jianjie.lu}@uni.kn %
%#########################################################################%
clearvars; clear global; close all; clc;

%% Initial Spatial discretization
a=0.5;
b=2.5;


%% Function Initialization
f1 = @(x) x.*(1+x.^2);
f2 = @(x) log(x);
f3 = @(x) -2*x.*exp(-x.^2);

%% Solution Initialization (we skip the +c on the indefinite integral)
F1 = @(x) 0.5*x.^2.*(1+0.5*x.^2);
F2 = @(x) x.*log(x)-x;
F3 = @(x) exp(-x.^2);

%% Error Computation and Error Plot for Midpoint-rule
fprintf('Midpoint Rule \n')
dx=0.1;
for i=1:6
    x = a:dx:b;
    I1 = midpointrule(x,dx,f1);
    error1_MP(i) = abs(F1(b)-F1(a)-I1);
    I2 = midpointrule(x,dx,f2);
    error2_MP(i) = abs(F2(b)-F2(a)-I2);
    I3 = midpointrule(x,dx,f3);
    error3_MP(i) = abs(F3(b)-F3(a)-I3);    
    convergence_rate_MP(i)=dx^2;
    fprintf('Error_1= %e -- Error_2= %e -- Error_3= %e -- dx= %e \n', ...
        error1_MP(i),error2_MP(i),error3_MP(i),dx)
    dx=dx/2.;
end

figure
semilogy(error1_MP,'bo-');
hold on
semilogy(error2_MP,'r+-');
semilogy(error3_MP,'kx-');
semilogy(convergence_rate_MP);
title('Midpoint-rule error convergence rate');
legend('Integral 1','Integral 2','Integral 3','Convergence rate');

%% Error Computation and Error Plot for trapezoidal-rule
fprintf('\nTrapezoidal Rule \n')
dx=0.1;
for i=1:6
    x = a:dx:b;
    I1 = trapezoidalrule(x,dx,f1);
    error1_TP(i) = abs(F1(b)-F1(a)-I1);
    I2 = trapezoidalrule(x,dx,f2);
    error2_TP(i) = abs(F2(b)-F2(a)-I2);
    I3 = trapezoidalrule(x,dx,f3);
    error3_TP(i) = abs(F3(b)-F3(a)-I3);  
    fprintf('Error_1= %e -- Error_2= %e -- Error_3= %e -- dx= %e \n', ...
        error1_TP(i),error2_TP(i),error3_TP(i),dx) 
    dx=dx/2.;
end

figure
semilogy(error1_TP,'bo-');
hold on
semilogy(error2_TP,'r+-');
semilogy(error3_TP,'kx-');
semilogy(convergence_rate_MP);
title('Trapezoidal-rule error convergence rate');
legend('Integral 1','Integral 2','Integral 3','Convergence rate');

%% Error Computation and Error Plot for simpson-rule
fprintf('\nSimpson Rule \n')
dx=0.1;
for i=1:6
    x = a:dx:b;
    I1 = simpsonrule(x,dx,f1);
    error1_S(i) = abs(vpa(F1(b)-F1(a))-I1); 
    % For the use of vpa(x) see Matlab Help.
    % Try also to remove it and see what happen.
    % Do not worry, unless it is specified, you do not have to take in
    % account round-off Matlab errors.
    I2 = simpsonrule(x,dx,f2);
    error2_S(i) = abs(F2(b)-F2(a)-I2);
    I3 = simpsonrule(x,dx,f3);
    error3_S(i) = abs(F3(b)-F3(a)-I3);  
    fprintf('Error_1= %e -- Error_2= %e -- Error_3= %e -- dx= %e \n', ...
        error1_S(i),error2_S(i),error3_S(i),dx) 
    convergence_rate(i)=dx^4;
    dx=dx/2.;
end

figure
plot(error1_S,'bo-');
hold on
title('Simpson-rule error integral 1');
figure
semilogy(error2_S,'r+-');
hold on
semilogy(error3_S,'kx-');
semilogy(convergence_rate);
title('Simpson-rule error convergence rate');
legend('Integral 2','Integral 3','Convergence rate');